# psm-test.tcl 
# Jungmin So (jso1@uiuc.edu)
# Modified by Matt Miller (mjmille2@uiuc.edu)

# ======================================================================
# Define options
# ======================================================================
set opt(chan)           Channel/WirelessChannel    ;# channel type
set opt(prop)           Propagation/TwoRayGround   ;# radio-propagation model
set opt(netif)          Phy/WirelessPhy            ;# network interface type
set opt(mac)            Mac/802_11M              ;# MAC type
set opt(ifq)            Queue/DropTail/PriQueue    ;# interface queue type
set opt(ll)             LL                         ;# link layer type
set opt(ant)            Antenna/OmniAntenna        ;# antenna model
set opt(ifqlen)         50                         ;# max packet in ifq
set opt(nn)             5                        ;# number of mobilenodes
set opt(flow)		1
set opt(adhocRouting)   DSDV                       ;# routing protocol

set opt(cp)             ""                         ;# connection pattern file
set opt(sc)     	""    			;# node movement file. 

set opt(x)      500                            ;# x coordinate of topology
set opt(y)      500                            ;# y coordinate of topology
set opt(seed)   0.0                            ;# seed for random number gen.
set opt(stop)   50                            ;# time to stop simulation
# set opt(psm) "ON";
set opt(energymodel) EnergyModel;
set opt(rxpower) 1.0;
set opt(txpower) 1.4;
set opt(idlepower) 0.83;
# New values
set opt(rtsopt) "OFF"
set opt(sleeppower) 0.13;
set opt(offpower) 0.043;
set opt(rxpower_low) 0.0125;
set opt(txpower_low) 0.01488;
set opt(idlepower_low) 0.01236;
set opt(sleeppower_low) 0.000016;
set opt(offpower_low) 0.000000043;
set opt(mac_low) Mac/MacLow;
# end new values
set opt(initialenergy) 100.0;
set opt(f_out) out.tr

proc getopt {argc argv} {
	global opt

	for {set i 0} {$i < $argc} {incr i} {
		set arg [lindex $argv $i]
		if {[string range $arg 0 0] != "-"} continue;

		set name [string range $arg 1 end]
		set opt($name) [lindex $argv [expr $i+1]]
	}
}

getopt $argc $argv

# ============================================================================
# check for boundary parameters and random seed
if { $opt(x) == 0 || $opt(y) == 0 } {
	puts "No X-Y boundary values given for wireless topology\n"
}
if {$opt(seed) > 0} {
	puts "Seeding Random number generator with $opt(seed)\n"
	ns-random $opt(seed)
}

# create simulator instance
set ns_   [new Simulator]

set tracefd  [open $opt(f_out) w]
$ns_ trace-all $tracefd
$ns_ use-newtrace

# Create topography object
set topo   [new Topography]

# define topology
$topo load_flatgrid $opt(x) $opt(y)

# create God
create-god [expr $opt(nn)]

set chan_1_ [new $opt(chan)]
set chan_2_ [new $opt(chan)]

#configure for mobilenodes
# NOTE: use the same netif type for now.
# Default values are set in tcl/lib/ns-mobilenode.tcl.
# The power values are customized as well
$ns_ node-config -adhocRouting $opt(adhocRouting) \
		-llType $opt(ll) \
		-macType $opt(mac) \
		-ifqType $opt(ifq) \
		-ifqLen $opt(ifqlen) \
		-antType $opt(ant) \
		-propType $opt(prop) \
		-phyType $opt(netif) \
		-channel $chan_1_ \
		-topoInstance $topo \
		-energyModel $opt(energymodel) \
		-rxPower $opt(rxpower) \
		-txPower $opt(txpower) \
		-idlePower $opt(idlepower) \
		-initialEnergy $opt(initialenergy) \
		-rtsOpt $opt(rtsopt) \
		-sleepPower $opt(sleeppower) \
		-offPower $opt(offpower) \
		-macTypeLow $opt(mac_low) \
		-phyLow $opt(netif) \
		-channelLow $chan_2_ \
		-rxPowerLow $opt(rxpower_low) \
		-txPowerLow $opt(txpower_low) \
		-idlePowerLow $opt(idlepower_low) \
		-sleepPowerLow $opt(sleeppower_low) \
		-offPowerLow $opt(offpower_low) \
		-agentTrace ON \
		-routerTrace ON \
		-macTrace ON 
	#	-movementTrace OFF


for {set j 0} {$j < $opt(nn)} {incr j} {
	set node_($j) [$ns_ node]
}

###########################################################


###########################################################

# Load scenario file
if { $opt(sc) == "" } {
	puts "*** NOTE: no scenario file specified."
        set opt(sc) "none"
} else {
	puts "Loading scenario file..."
	source $opt(sc)
	puts "Load complete..."
}

puts " "
puts "IEEE 802.11 TWO-RADIO TEST"
puts " "

# Tell all nodes when the simulation ends
for {set i 0} {$i < $opt(nn) } {incr i} {
    $ns_ at $opt(stop).0 "$node_($i) reset";
}

$ns_ at $opt(stop).0001 "stop"
$ns_ at $opt(stop).0002 "puts \"NS EXITING...\" ; $ns_ halt"

proc stop {} {
    global ns_ tracefd
    $ns_ flush-trace
    close $tracefd
}

# informative headers for CMUTracefile
#puts $tracefd "M 0.0 nn $opt(nn) x $opt(x) y $opt(y) rp \
#	$opt(adhocRouting)"
#puts $tracefd "M 0.0 sc $opt(sc) cp $opt(cp) seed $opt(seed)"
#puts $tracefd "M 0.0 prop $opt(prop) ant $opt(ant)"

puts "Starting Simulation..."
$ns_ run

